function cor_vec = datapre(tsdata)
% The routine for preparing time series.
% tsdata must be 3-D array, f * 90 * s, where f means frequency, 90
% means the brain region and size means the sample size. 
% Fisher transformation performed in the procedure on the correlation.
%
% Value,
%   a matrix with 4005 * s. The order of links is the uptriangle of
%   correlation matrix.
%
%% Gan Xinjun, Fudan University, CCSB.
% xjgansdu@163.com, 2013.11.29 

% check the dimension.
dim = ndims(tsdata);
if abs(dim - 3) > eps
    error('Check your data please, the dimension of time series must be 3');
end

[f,b,s] = size(tsdata);

if abs(b - 90) > eps
    error('Check your data please, the dimension of Brain is not 90');
end

% coordinate of the upper triangle 
ind = 1:b*(b-1)/2; % the length of the element in upper triangle
% ind2sub for upper triangle
J = round(floor(-.5 + .5 * sqrt(1 + 8 * (ind - 1))) + 2);
I = round(J .* (3 - J) / 2 + ind - 1);
% sub2ind for the 90 times 90 matrix
IND = sub2ind([b,b],I,J);

% calculate the correlation.
cor_vec=[]; 
h = waitbar(0,'Please wait...');
for i=1:s
    cor = corr(tsdata(:,:,i));
    cor_vec = [cor_vec;cor(IND)];
    waitbar(i/s,h)
end
delete(h);        
cor_vec = 1/2*log((1+cor_vec)./(1-cor_vec)); % Fisher transformation
