function output = fdrbh(dat, grp, threshold, vartype)
%The program for the FDR Benjamini-Hochberg of the Dataset
% also see mafdr.m
% Arguments
%   dat, the matrix with row as links and column as samples.
%   grp, the group labels for the samples. For example, the dat is 4005 *
%       50 and the 1:25 are patients, 26:50 are controls, then grp can be
%       defined as [zeros(25,1);ones(25,1)];
%   threshold, the threshold for identifying significant links, default 0.2
%   vartype, two-sample t test with 'equal' or 'unequal' variance, default
%       'unequal'.
% Value
%   output, a struct,
%       .fdr, the FDR for the links
%       .stat, the statistics for the links
%       .pvalue, the p-values for the links
%       .th, the matrix for result with fdr < q.
%
%% Gan Xinjun, Fudan University, CCSB
% xjgansdu@163.com
% 
[d,n] = size(dat);
lg = length(grp);
if abs(n-lg) > eps
    error('The size of data and group are not consistent');
end

% type is Eq of Neq for the variance
if (~exist('vartype','var') || isempty(vartype))
    vartype = 'N';   % default as Neq
end

if (~exist('threshold','var') || isempty(threshold))
    threshold = 0.05;   % default as 1
end


    [t, p, df]= hypotest(dat, grp, vartype);
    m = numel(p);
    [p_ord, idx] = sort(p);% [sorted_pvals,sort_index] = sort(pvals);
    fdr_ord =  p_ord(:) .* (m./(1:m))';   
% Running min in reverse order (in-place)
    fdr = zeros(m,1);
    for i = 1:m
        idx_min = min(find(p_ord>=p(i)));  
        if isempty(idx_min)
           fdr(i) = 1;
        else
           fdr(i) = min(fdr_ord(idx_min:m));  %-"Adjusted" p-values
        end
    end
    output.stat = t;
    output.pvalue = p;
    output.fdr = fdr;
    plotfdrbh(p,fdr);
    index = fdr < threshold;
    vec = (1:d)';
    output.th = sortrows([t(index),p(index),fdr(index),vec(index)],2);

