function [t, p, df]= hypotest(dat, grp, vartype)
%The program for two-sample t test.
% Arguments
%   dat is a matrix of data. row is variable while column is sample.
%   grp describes the group of the data, such as Patient with Control.
% Values
%   t, the t-statistic
%   p, p-values
%   df, the dgree of freedom
%% Gan Xinjun, Fudan University, CCSB
% xjgansdu@163.com
%


% check the dimension.
[d,n] = size(dat);
lg = length(grp);
if abs(n-lg) > eps
    error('The size of data and group are not consistent');
end

% type is Eq of Neq for the variance
if (~exist('vartype','var') || isempty(vartype))
    vartype = 'N';   % default as Neq
end

label = unique(grp);     % identify the group
if abs(length(label)-2) > eps   % check the label of group
    error('The group should only consist of two elements')
end
IND1 = grp==label(1);  % coordinate of the first group, like Patient
IND2 = ~IND1;  % coordinate of the second group, like Control
X1 = dat(:,IND1);  
X2 = dat(:,IND2);

if strcmp(vartype,'N')
    % t test for unequal variance
    [~,p,~,stats] = ttest2(X1, X2, [],[],'unequal',2);  
else 
    % t test for equal variance
    [~,p,~,stats] = ttest2(X1, X2, [],[],'equal',2);
end
t = stats.tstat;
df = stats.df;


