function plotsam(stat_ob, stat_perm, sam_fdr, delta)
%% PLOTSAM yields a plot summarizing the SAM results.
%
% plotsam(stat_ob, stat_perm, sam_fdr, delta) computes the observed D values (stat_ob) and
% plots them against stat_perm , the expected D values. It finds the expressed
% observations on the basis of the thresholds detained by VEC_FALSE.
%
%% copy from Eric NIMPAYE, UAE-ENSAT-MISC, 2013.

d_sort = sort (stat_ob);

pl_title = sprintf ('SAM Plot for Delta = %.1f', delta);
x_label = 'Expected d(i) values';
y_label = 'Observed d(i) values';

%mat_fdr = stats_cal (d, d_bar, vec_false, delta);

%cutup
d_up = find (d_sort >= sam_fdr (1, 7));

%cutlow
d_low = find (d_sort <= sam_fdr (1, 6));

figure

if isempty (d_up) && isempty (d_low)
    plot (stat_perm, d_sort, '.k')
    title (pl_title)
    
else
    sz_d = length (d_sort);
    
%     len_up = length (d_up);
%     len_low = length (d_low);
%     excl_ind = zeros (1, len_up + len_low);
    
%     for i = 1:len_up
%         excl_ind (i) = d_up (i);
%     end
%     for i = 1:len_low
%         excl_ind (i + len_up) = d_low (i);
%     end
%     
    excl_ind = [d_up;d_low]';
%     mid_ind = zeros (1, sz_d - length (excl_ind));
%     j = 1;
%     for i = 1:sz_d
%         if sum (excl_ind == i) == 0
%             mid_ind (j) = i;
%             j = j + 1;
%         end
%     end
    mid_ind = setdiff((1:sz_d),excl_ind);
    plot (stat_perm (mid_ind), d_sort (mid_ind), '.k');
    title (pl_title)
    
    hold on
    plot (stat_perm (d_up), d_sort (d_up), 'diamondr');
    
    hold on
    plot (stat_perm (d_low), d_sort (d_low), 'squarer');
end

xlabel (x_label)
ylabel (y_label)

hold on
buff_refl = refline (1, 0);
set (buff_refl, 'Color', 'g');

hold on
buff_refl = refline (1, delta);
set (buff_refl, 'Color', 'k', 'LineStyle', '--');

hold on
buff_refl = refline (1, -delta);
set (buff_refl, 'Color', 'k', 'LineStyle', '--');

if ~isinf (sam_fdr (1, 7))
    hold on
    buff_refl = refline ([0, sam_fdr(1,7)]);
    set (buff_refl, 'Color', 'm', 'LineStyle', '-.');
end

if ~isinf (sam_fdr (1, 6))
    hold on
    buff_refl = refline ([0, sam_fdr(1,6)]);
    set (buff_refl, 'Color', 'm', 'LineStyle', '-.');
end
