function [f0fz, xbreaks] = smooth1d(Z, zstar, nr)
%% The smoothing procedure for fdr1d
%% Gan Xinjun, Fudan University, CCSB
%

% the region
xbreaks = MidBreaks(Z, nr);


%% smoothing procedure
count = hist(Z, xbreaks)';
scount = smooth1d_direct(count);
fz = scount/(length(Z) * (xbreaks(2) - xbreaks(1)));
zstar = zstar(xbreaks(1) <= zstar & zstar <= xbreaks(nr));
count = hist(zstar, xbreaks);
scount = smooth1d_direct(count);
f0 = scount/(length(zstar) * (xbreaks(2) - xbreaks(1)));
f0fz = f0./fz;


function breaks = MidBreaks(x, nr)
%% Matlab version of MidBreaks in OCplus
mids = linspace(min(x), max(x), nr - 1)';
d = mids(2) - mids(1);
breaks = [mids - d/2; mids(nr - 1) + d/2];


function [fit, df]= smooth1d_direct(y, sv2, err)
%% matlab version of smooth1d in OCplus
if (~exist('sv2','var') || isempty(sv2))
    sv2 = 0.01;   
end

y = y(:);
if (~exist('err','var') || isempty(err))
    err = 1e-4;   
end

n = length(y);
R1 = Rinv(n, 2);
beta = log(mean(y));
b = zeros(n,1);
lambda = 1/sv2;
oldval = 2 * lambda;
while (abs(oldval - lambda)/lambda > err & lambda < 1/err^2) 
    oldval = lambda;
    eta = beta + b;
    p = exp(eta);
    eps = (y - p)./(p + 1e-06);
    Y = eta + eps;
    wt = p;
    
    % b = solve(diag(wt) + R1/sv2, wt .* (Y - beta))
    b = (diag(wt) + R1/sv2)\(wt .* (Y - beta)); 
    eta = beta + b;
    p = exp(eta);
    eps = (y - p)./(p + 1e-06);
    Y = eta + eps;
    wt = p;
    beta = sum(wt .* (Y - b))./sum(wt);
    brb = squeeze(b' * R1 * b);
    wtrs = diag(wt) + R1/sv2;
    % if rcond(wtrs) < 1e-15
    %    break;
    %    sv2 = NaN;
    % end
    trc = sum(sum(wtrs^(-1) .* R1));
    sv2 = (brb + trc)./(n - 2);
    df = sum(diag(wtrs^(-1)) .* wt);
    lambda = 1/sv2;
end
fit = p;

function Ri = Rinv(n, d)
%% matlab version of Rinv in OCplus
delta = diag(-ones(n-1,1)) + diag(ones(n-2,1),1);
delta = [delta [zeros(n-2,1);1]];

cnt = 1;

while cnt < d
    delta = delta(2:end, cnt+1:end) * delta;
    cnt = cnt + 1;
end

Ri = delta' * delta;
