function [b, xbreaks, ybreaks]= smooth2d(Z, zstar, nr, smoothp)
%% smoothing procedure for fdr2d
% f0fz = b./((1 - b) * nperm);
%% Gan Xinjun, Fudan University, CCSB
%

% The region
xbreaks = midbreakst(Z(:,1), nr);
ybreaks = midbreakst(Z(:,2), nr);

%% smoothing procedure
edges{1} = xbreaks;
edges{2} = ybreaks;
tabstat = hist3([Z;zstar],'Edges',edges);
tabnull = hist3(zstar,'Edges',edges);
df_smooth = (1 - smoothp) * (nr - 1)^2;
b = smooth2d_direct(tabnull, tabstat, df_smooth);


function [pfit, df, sv2] = smooth2d_direct(y, n, df)
%% The logistic procedure to smoothing Z and zstar
% matlab version of smooth2d.direct in OCplus
% obtain the fit r
if (~exist('df','var') || isempty(df))
    df = 100;   
end
ry = size(y,1);
n(n < 1) = 1;
p = y./n;

ff = @(x)dgfree(ry,x) - df;
sv2 = fzero(ff,[0.0001,1000]);
sv2 = sv2(1);
pfit = smooth2d_basic(p,sv2); 



function xbreaks = midbreakst(x, nr, eps, iter)
%% matlab version of the function Midxbreak.t in OCplus
if (~exist('eps','var') || isempty(eps))
    eps = 0.01;   
end
if (~exist('iter','var') || isempty(iter))
    iter = 20;   
end

if max(x) < 0 | min(x) > 0
    error('The range of the statistics must negative and positive')
end
rr = [min(x);max(x)];
lower = eps;
upper = max(x) - min(x) + 2 * eps;
stepi = 1;
cur_cnt = nr - 3;
while (stepi < iter & cur_cnt ~= nr - 2) 
    cur_d = (lower + upper)/2;
    left = ceil((-rr(1) + eps)/cur_d - 0.5);
    right = ceil((rr(2) + eps)/cur_d - 0.5);
    cur_cnt = left + right;
    if cur_cnt > nr - 2
        lower = cur_d;
    elseif cur_cnt < nr - 2
        upper = cur_d;
    end
    stepi = stepi + 1;
end
cur_work = cur_d;
sub_good = 0;
sub_work = eps;
stepi = 1;
while stepi < iter 
    cur_d = cur_work - sub_work;
    left = ceil((-rr(1) + eps)/cur_d - 0.5);
    right = ceil((rr(2) + eps)/cur_d - 0.5);
    cur_cnt = left + right;
    if cur_cnt == nr - 2
        sub_good = sub_work;
        sub_work = 2 * sub_work;
    else 
        sub_work = (sub_work + sub_good)/2;
    end
    stepi = stepi + 1;
end
cur_d = cur_work - sub_good;
left = ceil((-rr(1) + eps)/cur_d - 0.5);
right = ceil((rr(2) + eps)/cur_d - 0.5);

ret_left = -cur_d/2 + (0:-cur_d:-cur_d * left)';
ret_right = cur_d/2 + (0:cur_d:cur_d * right)';
xbreaks = sort([ret_left; ret_right]);


function df = dgfree(nx, sv2, ny)
%% matlab version of dgfree in OCplus
if (~exist('ny','var') || isempty(ny))
    ny = nx;   
end
imp = zeros(nx,ny);
imp(nx/2, ny/2) = 1;
impresp = smooth2d_basic(imp,sv2); 
df = nx.*ny.*impresp(nx/2, ny/2);


function est = smooth2d_basic(y, sv2, se2, err)
%% matlab version of smooth2d.basic in OCplus
if (~exist('sv2','var') || isempty(sv2))
    sv2 = 0.01;   
end
if (~exist('se2','var') || isempty(se2))
    se2 = 1;   
end
if (~exist('err','var') || isempty(err))
    err = 0.01;   
end

W = 1/se2;
ry = size(y,1);
cy = size(y,2);
Y = y(:);
[loc,num] = rmatrix(ry,cy);
d = num/sv2 + W;   
xx = loc;
ybar = mean(Y);
B =  W .*(Y-ybar);  %  # Z'W (Y-Xb) 
v =  gausei(d, xx, B, sv2, err);
est = ybar+v;
est = reshape(est,length(est)/cy,cy);%matrix(est,ncol=ny)



function [loc,num] = rmatrix(nx,ny)
%% Matlab version of rmatrix in OCplus
grid=((1:nx)'*ones(1,ny)-1)*1000+ ones(nx,1)*(1:ny)-1;   %    grid = c((row(xymat)-1)*1000 + col(xymat)-1)
grid = grid(:);
neighbor=[grid+1000;grid-1000;grid+1;grid-1];  %neighbor= c(grid+1000,grid-1000,grid+1,grid-1)
[~,loc] = ismember(neighbor,grid);
loc = reshape(loc,length(loc)/4,4);
num = (loc>0)*ones(4,1);
num = num(:);   


function  vv=gausei(d, xx, B, sv2, err, maxiter)
%% matlab version of gauser in OCplus
if (~exist('maxiter','var') || isempty(maxiter))
    maxiter = 100;   
end
nd = length(d);             %         # number of grid points
xx=xx(:);
xx(xx < 1) = nd+1;
v = [B./(d+0.000001);0];  
vold = 3 * v;
stepiter = 1;
while ((std(vold-v)/std(v) > err) && (stepiter < maxiter))
    vold = v;
    vx = v(xx);
    vmat=reshape(vx,length(vx(:))/4,4);
    v(1:nd)=( B+ vmat * ones(4,1)/sv2)./d;
    stepiter = stepiter+1;
end  
vv=v(1:nd);

