function output = ttest2d(xdat,grpx, zdat, grpz, vartype)
%% The t-test for 2-D datasets
%
% Values
%   output, a struct
%       .tx, the t-statistic for xdat
%       .tz, the t-statistic for zdat
%       .px, the p-value for xdat
%       .pz, the p-value for zdat
%
%% Gan Xinjun, Fudan University, CCSB
%


%% deal with xdat
label = unique(grpx);
if length(label) ~= 2
    error('Check your grp label please, not 2 groups');
end
%% t-score and p-value 
X1 = xdat(:,grpx == label(1));
X2 = xdat(:,grpx == label(2));
ngrp1 = size(X1,2);
ngrp2 = size(X2,2);

% Return group means var means and t scores. 

seg1 = var(X1,[],2);
seg2 = var(X2,[],2); 
grpmean_diff = mean(X1,2)-mean(X2,2);

if strcmp(vartype,'N')
    %==Two-sample assuming unequal variance t-test.
    sg1 = seg1./ngrp1;
    sg2 = seg2./ngrp2;
    grpse = sqrt(sg1 + sg2); 
    df = double((sg1+sg2).^2 ./ (sg1.^2 ./(ngrp1-1) + sg2.^2 ./(ngrp2-1)));
else
    %==Two-sample assuming equal variance t-test.
    df = ngrp1 + ngrp2 - 2;
    pooled_se = sqrt(((ngrp1-1) .* seg1 + (ngrp2-1) .* seg2) ./ df);
    grpse = pooled_se .* sqrt(1./ngrp1 + 1./ngrp2);
end
tx = double(grpmean_diff ./ grpse);
if ~isscalar(tx) && ~strcmp(vartype,'N')
    df = repmat(df,size(tx));
end
px = 2 * tcdf(-abs(tx),df);


%% deal with zdat
% if zdat does not exist.
if (~exist('zdat','var') || isempty(zdat))
    zdat = xdat;   
    grpz = grpx;
end

label = unique(grpz);
if length(label) ~=2
    error('Check your grp label please, more than 2 groups');
end
%% t-score and p-value 
Z1 = zdat(:,grpz == label(1));
Z2 = zdat(:,grpz == label(2));
ngrp1 = size(Z1,2);
ngrp2 = size(Z2,2);

% Return group means var means and t scores. 

seg1 = var(Z1,[],2);
seg2 = var(Z2,[],2); 
grpmean_diff = mean(Z1,2)-mean(Z2,2);

if strcmp(vartype,'N')
    %==Two-sample assuming unequal variance t-test.
    sg1 = seg1./ngrp1;
    sg2 = seg2./ngrp2;
    grpse = sqrt(sg1 + sg2); 
    df = double((sg1+sg2).^2 ./ (sg1.^2 ./(ngrp1-1) + sg2.^2 ./(ngrp2-1)));
else
    %==Two-sample assuming equal variance t-test.
    df = ngrp1 + ngrp2 - 2;
    pooled_se = sqrt(((ngrp1-1) .* seg1 + (ngrp2-1) .* seg2) ./ df);
    grpse = pooled_se .* sqrt(1./ngrp1 + 1./ngrp2);
end

tz = double(grpmean_diff ./ grpse);
if ~isscalar(tz) && ~strcmp(vartype,'N')
    df = repmat(df,size(tz));
end
pz = 2 * tcdf(-abs(tz),df);

output.tx = tx(:);
output.tz = tz(:);
output.px = px(:);
output.pz = pz(:);
